#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* mpirunPooch.c - parallel application launcher 
From OS X's Unix command line, mpirunPooch uses Pooch to launch parallel 
MacMPI_X-type jobs while emulating the classic mpirun of mpich.  
Compile using: sudo cc mpirunPooch.c -o /usr/bin/mpirun

Copyright  2001-5  Dean E. Dauger, Dauger Research, Inc.  http://daugerresearch.com/

This code requires Pooch version 1.6 or later to be installed on the local 
network of Macs.  It translates the parameters entered on the Unix command 
line into AppleScripts that communicate with Pooch.  A download version 
of Pooch is available at: http://daugerresearch.com

****************************************************************
License:

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU General Public License is available by:
Writing to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
or accessing http://www.fsf.org/copyleft/gpl.html

****************************************************************

Version History:

2005 June 20 - Added -T lam for LAM/MPI

2004 August 17 - Added -T mpichgm for Myrinet version of mpich

2004 August 14 - Added -r for retrieval and -p for tasks per computer

2003 June 18 - Factored code for reuse by others in macmpi suite

2002 October 22 - Added mpirun-like options, number of processor option, and job type option

2002 May 21 - Added -W, -, and --help options and changed the makefile 

2001 October 3 - Creation (as plaunch.c) using cc on Mac OS X 10.1 

Thanks goes to the following for advice and recommendations:
  Daniel Gordon (NRL), Mark Chavira (UCLA EE/CS), 
  Charles Norton (NASA JPL), Viktor Decyk (UCLA Physics)
*/


void printUsage(char *myName); 
void printUsage(char *myName)
	{
	printf("\n"); 
	printf("Usage: \n\n"); 
	printf("%s executable [file1 file2 ...] -np nodeCount\t! searches for processors on the \"best\" nodes\n", myName);
	printf("%s executable [file1 file2 ...] -n addr1 addr2 ...\n", myName); 
	printf("%s -j jobFile", myName);
	printf("\t\t! \"%s -j\" prints a sample jobFile\n", myName);
	printf("\nadditional options: \n"); 
	printf("\t -s : save the AppleScript file into a file named /tmp/tempScript\n"); 
	printf("\t -m  (or)  -nolocal : not me! Don't include me in the node list by default\n"); 
	printf("\t -allcpus : include every cpu and node Pooch can find\n"); 
	printf("\t -d : select a directory for the files to be on the remote nodes\n"); 
	printf("\t -t : test mode on: prints out what mpirun will do, rather than doing it \n"); 
	printf("\t -T : select a MPI type for the job: -T MacMPI, -T mpich, -T MPIPro, -T mpichgm, -T lam \n"); 
	printf("\t -r : retrieve output: -r 0, -r all, -r off\n"); 
	printf("\t -p : number of tasks per computer (0 = use proc count)\n"); 
	printf("\t -tcp : use a particular TCP port number (0 = use default)\n"); 
	printf("\t -K : skip the check for nodes being BUSY and launch anyway\n"); 
	printf("\t -Q : in case of a launch failure, Pooch queues the job for later relaunch\n"); 
	printf("\t -W : wait for user to select additional nodes or files and launch the job manually\n"); 
	printf("\t - or --help : print usage information\n"); 
	printf("\n"); 
	}

#define ForegroundMode			1

#include "fileutil.c"

#include "poochport.c"

#include "poochcljobinput.c"

#include "poochjobexecute.c"

int main(int argc, char *argv[])
	{int out=0; 
	JobOptionsStruct o; 

	if (!parseJobArgumentsFromMainArgs(argc, argv, &o)) printUsage(argv[0]); 
	else {
		int i;
		
		if (o.proceed) 
			executepoochlaunch(&o); 
		
    	}

	return out; 
	}
