/* 
Copyright  2003  
   Dauger Research, Inc. 
   http://daugerresearch.com/
*/

/*
routines to call the poochportd, which forwards requests to Pooch running in another user space
*/

#ifndef __poochport

#define __poochport

#include "fileutil.c"

#include "poochport.h"

char *callosascriptpoochportd(const char *scriptstr)
	{char *reply=NULL; 
	char *portstr; 
	short verbose=0; 
	
	portstr=readfromfile(PortNamePathName); 
	
	if (portstr) {int callingsocket; 
		if (verbose) printf("Port string: %s\n", portstr); 
		
		callingsocket = socket(AF_UNIX, SOCK_STREAM, 0); 
		if (verbose) printf("socket returned %d\n", callingsocket); 
		
		if (callingsocket) {
			int err; 
			struct sockaddr_un serveraddr; 
			
			strcpy(serveraddr.sun_path, portstr);
			serveraddr.sun_family = AF_UNIX; 
			if (verbose) printf("connecting...\n"); 
			err=connect(callingsocket, (struct sockaddr *)&serveraddr, 
				serveraddr.sun_len = strlen(serveraddr.sun_path) 
				+ sizeof(serveraddr.sun_len) 
				+ sizeof(serveraddr.sun_family)); 
			if (verbose) printf("connect returned %d\n", err); 
			
			if (!err) {PoochPortHeaderStruct header={ScriptCommand,0}; 
				PoochPortHeaderStruct replyHeader; 
				int data[4]={0x03314159,0,0,0}; 
				
				header.type=ScriptCommand; 
				header.length=strlen(scriptstr)+1; 
				err=send(callingsocket, &header, sizeof(header), 0); 
				if (verbose) printf("send returned %d\n"); 
				err=send(callingsocket, scriptstr, header.length, 0); 
				if (verbose) printf("send returned %d\n"); 
				err=recv(callingsocket, &replyHeader, sizeof(replyHeader), 0); 
				data[0]=replyHeader.type; data[1]=0; 
				if (verbose) printf("recv'ed %s, err = %d\n", data, err); 
				if (replyHeader.length>0) {
					reply=(char*)malloc(replyHeader.length); 
					if (reply) {
/*						err=recv(callingsocket, reply, replyHeader.length, 0); */
						long amountReceived=0; 
						do {
							err=recv(callingsocket, 
								reply+amountReceived, 
								replyHeader.length-amountReceived, 0); 
							if (err>0) 
								amountReceived+=err; 
							} 
							while ((err>0)&&(amountReceived<replyHeader.length)); 
						
						if (verbose) printf("recv'ed %s, err = %d\n", reply, err); 
						}
					}
								
				}		
			
			shutdown(callingsocket,2); 
			close(callingsocket); 
			}
		
		free(portstr); 
		}
	
	return reply; 
	}
	
char *callopenpoochportd(const char *filepathstr)
	{char *reply=NULL; 
	char *portstr; 
	short verbose=0; 
	
	portstr=readfromfile(PortNamePathName); 
	
	if (portstr) {int callingsocket; 
		if (verbose) printf("Port string: %s\n", portstr); 
		
		callingsocket = socket(AF_UNIX, SOCK_STREAM, 0); 
		if (verbose) printf("socket returned %d\n", callingsocket); 
		
		if (callingsocket) {
			int err; 
			struct sockaddr_un serveraddr; 
			
			strcpy(serveraddr.sun_path, portstr);
			serveraddr.sun_family = AF_UNIX; 
			if (verbose) printf("connecting...\n"); 
			err=connect(callingsocket, (struct sockaddr *)&serveraddr, 
				serveraddr.sun_len = strlen(serveraddr.sun_path) 
				+ sizeof(serveraddr.sun_len) 
				+ sizeof(serveraddr.sun_family)); 
			if (verbose) printf("connect returned %d\n", err); 
			
			if (!err) {PoochPortHeaderStruct header={OpenCommand,0}; 
				PoochPortHeaderStruct replyHeader; 
				int data[4]={0x03314159,0,0,0}; 
				
				header.type=OpenCommand; 
				header.length=strlen(filepathstr)+1; 
				err=send(callingsocket, &header, sizeof(header), 0); 
				if (verbose) printf("send returned %d\n"); 
				err=send(callingsocket, filepathstr, header.length, 0); 
				if (verbose) printf("send returned %d\n"); 
				{PoochPortHeaderStruct replyHeader; 
				err=recv(callingsocket, &replyHeader, sizeof(replyHeader), 0); 
				data[0]=replyHeader.type; data[1]=0; 
				if (verbose) printf("recv'ed %s, err = %d\n", data, err); 
				if (replyHeader.length>0) {
					reply=(char*)malloc(replyHeader.length); 
					if (reply) {
						err=recv(callingsocket, reply, replyHeader.length, 0); 
						if (verbose) printf("recv'ed, err = %d\n", err); 
						}
					}
				else
					{
					reply=(char *)malloc(1); 
					if (reply) 
						reply[0]=0; 
					}
				}
								
				}		
			
			shutdown(callingsocket,2); 
			close(callingsocket); 
			}
		
		free(portstr); 
		}
	
	return reply; 
	}

#include <sys/param.h>
char *getfullpathstr(char *infilestr, char *indirpath)
	{char *out=NULL; 
	
	if (infilestr) {
		
		if (strlen(infilestr)) 
		if ('/'==infilestr[0]) {//already absolute
			out=(char*)malloc(strlen(infilestr)+2); 
			if (out) {
				strcpy(out, infilestr); 
				}
			}
		else {
			char *dirpath=NULL; 
			if (!indirpath) {
				system("pwd >> /tmp/mydirectorypath.txt"); 
				dirpath=readfromfile("/tmp/mydirectorypath.txt"); 
				}
			else {
				dirpath=malloc(strlen(indirpath)+1); 
				strcpy(dirpath, indirpath); 
				}
			if (dirpath) {
				char *homepathstr=NULL; 
				remove("/tmp/mydirectorypath.txt"); 
				switch(infilestr[0]) {
					default: out=(char*)malloc(strlen(dirpath)+strlen(infilestr)+2); 
						if (out) {
							strcpy(out, dirpath); 
							if ('\n'==out[strlen(out)-1]) 
								out[strlen(out)-1]=0; //replace newline char
							if ('/'!=out[strlen(out)-1]) 
								strcat(out, "/"); //add / char
							strcat(out, infilestr); 
							}
						break; 
					case '~': homepathstr=getenv("HOME"); 
						if (homepathstr) {
							out=(char*)malloc(strlen(homepathstr)+strlen(infilestr)+1); 
							if (out) {
								switch (infilestr[1]) {
									case 0: 
										strcpy(out, homepathstr); 
										break; 
									case '/': 
										strcat(strcpy(out, homepathstr), &infilestr[2]); 
										break; 
									default: 
										printf("Sorry, path %s not recognized\n", infilestr); 
										break; 
										}
								
								}
							}
						break; 
					case '/': //already absolute
						out=(char*)malloc(strlen(infilestr)+2); 
						if (out) {
							strcpy(out, infilestr); 
							}
						break; 
					}
				
				{char *fullpathstr=out; 
				out=malloc(MAXPATHLEN); 
				out[0]=0; 
				if (realpath(fullpathstr, out)) 
					free(fullpathstr); 
				else {//put it back
					free(out); 
					out=fullpathstr; 
					}
				}
				
				if (dirpath) free(dirpath); 
				dirpath=NULL; 
				}
			}
		}
	
	return out; 
	}

int poochopenthis(char *infilestr, char *indirpath)
	{int out=0; 
	if (infilestr) {
		if (strlen(infilestr)) {
			char *fullpathstr=getfullpathstr(infilestr, indirpath); 
			
			if (fullpathstr) {
				char *reply=callopenpoochportd(fullpathstr); 
				if (reply) {
					out=1; 
					free(reply); 
					}
				free(fullpathstr); 
				}
			
			}
		}
	
	return out; 
	}

#endif
