#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* poochstat.c - accesses node information about a Pooch cluster
From OS X's Unix command line, poochstat uses Pooch to scan for 
nodes or jobs and returns interesting information about them.  
Compile using: sudo cc poochstat.c -o /usr/bin/mpistat

Copyright  2003-5  Dean E. Dauger, Dauger Research, Inc.  http://daugerresearch.com/

This code requires Pooch version 1.6 or later to be installed on the local 
network of Macs.  It translates the parameters entered on the Unix command 
line into AppleScripts that communicate with Pooch.  A download version 
of Pooch is available at: http://daugerresearch.com

****************************************************************
License:

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU General Public License is available by:
Writing to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
or accessing http://www.fsf.org/copyleft/gpl.html

****************************************************************

Version History:

2005 June 20 - New scan options and parsing routines for job data

2004 August 13 - Addition of BUSY nodes and new scan options

2003 May 9 - Creation using cc on Mac OS X 10.2.5

Thanks goes to the following for advice and recommendations:
  Charles Norton (JPL), Viktor Decyk (UCLA Physics)
*/


void printUsage(); 
void printUsage()
	{
	printf("Note: Pooch may take up to 8 seconds to perform its scan for nodes.\n"); 
	printf("Usage: \n\n"); 
	printf("\t -s : save the AppleScript file into a file named /tmp/tempScript\n"); 
	printf("\t -t : test mode on: prints out what I will do, rather than doing it \n"); 
	printf("\t -w : print out all information in wide mode\n"); 
	printf("\t -a [IP Address] : perform the node scan starting at a remote address\n"); 
	printf("\t -n [count] : scan for the best nodes it can find, up to count\n"); 
	printf("\t -b : include BUSY nodes\n"); 
	printf("\t -j : scan for jobs rather than nodes\n"); 
	printf("\t -g secs : when scanning for jobs, look back secs ago\n"); 
	printf("\t -f : use a fresh, new scan\n"); 
	printf("\t -r : print out raw reply data from script rather than line format\n"); 
	printf("\t - or --help : print usage information\n"); 
	printf("\n"); 
	}

#include "poochnodescan.h"

typedef struct OptionsStruct {
	short proceed; 
	short saveScript; 
	short wideMode; 
	short testMode;
	short rawForm; 
	short includeBUSY; 
	short useNewScan; 
	short jobScan; 
	int remoteNodeScanArg; 
	int bestCount; 
	long flags; 
	unsigned long recallTime; 
	int numAvailableNodes; 
	int numJobs; 
	NodeExtendedPtr availableNodeList; 
	int unused; 
	} OptionsStruct, *OptionsPtr, **OptionsHandle; 

int parseArguments(int argc, char *argv[], OptionsPtr optionsP); 
int parseArguments(int argc, char *argv[], OptionsPtr optionsP)
	{int out=0; 
	
if (optionsP) {
	{long *tp=(long*)optionsP, i=sizeof(OptionsStruct)>>2; 
	while (i--) tp[i]=0; /* clear it */
	}
	
	if (argc<=0) optionsP->proceed=0; 
	else {
		int i; 
		optionsP->proceed=out=1; 
		
		for(i=1; i<argc; i++) {/* item by item */
			short unrecognizedOption=0, notADashOption=0; 
			
			if ('-'==argv[i][0]) {/* an option */
				char *optS=&argv[i][1]; 
				
				if (strlen(optS)>1) {
#define	ifOptIs(str)	if(!strcmp(optS,str))
					
					ifOptIs("help") {//print help
						optionsP->proceed=0; 
						out=0; 
						}
					else {
						unrecognizedOption=1; 
						printf("Unrecognized option: %s\n", optS); 
						}
					
#undef ifOptIs
					}
				else //single letter or null 
					switch (optS[0]) {
						case '-': //--
						case 0: //-
						case 'h': //-help
						case '?': //-?
							optionsP->proceed=0; 
							out=0; 
							break; 
						case 's': optionsP->saveScript=1; break; 
						case 'w': optionsP->wideMode++; break; 
						case 't': optionsP->testMode=1; break; 
						case 'b': optionsP->includeBUSY=1; break; 
						case 'f': optionsP->useNewScan=1; break; 
						case 'j': optionsP->jobScan=1; break; 
						case 'r': optionsP->rawForm=1; break; 
						case 'a': case '@': 
							if ((i+1<argc)&&!optionsP->remoteNodeScanArg) 
								optionsP->remoteNodeScanArg=i+1; 
							break; 
						case 'g': 
							if ((i+1<argc)&&!optionsP->recallTime) {
								sscanf(argv[i+1], "%d", &optionsP->recallTime); 
								if (!optionsP->recallTime) {
									if (!strcmp("minute", argv[i+1])) 
										optionsP->recallTime=60L; 
									else if (!strcmp("hour", argv[i+1])) 
										optionsP->recallTime=3600L; 
									else if (!strcmp("day", argv[i+1])) 
										optionsP->recallTime=3600L*24; 
									else if (!strcmp("week", argv[i+1])) 
										optionsP->recallTime=3600L*24*7; 
									else if (!strcmp("month", argv[i+1])) 
										optionsP->recallTime=3600L*24*31; 
									 
									}
								}
							break; 
						case 'n': 
							if ((i+1<argc)&&!optionsP->bestCount) 
								sscanf(argv[i+1], "%d", &optionsP->bestCount); 
							break; 
						default: unrecognizedOption=1; 
							printf("Unrecognized option: %s\n", optS); 
							break; 
					}
				}
			else 
				notADashOption=1; 
			
			if (unrecognizedOption||notADashOption) {
				/*
				if (optionsP->remoteNodeScanArg) {
					optionsP->remoteNodeScanArg=i; 
					}
				*/
				
				}
			
			}
		}
	
	}
	
	return out; 	
	}



#include "poochport.c"

#include "poochnodescan.c"

int main(int argc, char *argv[])
	{int out=0; 
	OptionsStruct o; 

	if (!parseArguments(argc, argv, &o)) printUsage(); 
	else {
		int i;
		
		if (o.proceed) 
			{
							
			if (o.proceed) {//write out the AppleScript(s) and run them
				FILE *fp=fopen("/tmp/tempScript", "w"); 
				if (fp) {
					rewind(fp); 
					fprintf(fp, "with timeout of 16 seconds\ntell application \"Pooch App\"\n"); 
				//	fprintf(fp, " activate\n"); 
					fprintf(fp, o.jobScan?" get job scan":" get node scan"); 
					
					if (o.remoteNodeScanArg) 
						fprintf(fp, " starting at \"%s\"", 
							argv[o.remoteNodeScanArg]); 
					
					if (o.bestCount) 
						fprintf(fp, " for best %d", 
							o.bestCount); 
					
					if (o.includeBUSY) 
						fprintf(fp, " with BUSY"); 
					
					if (o.useNewScan) 
						fprintf(fp, " with new scan"); 
					
					if (o.recallTime) 
						fprintf(fp, " recalling %d", o.recallTime); 
					
					fprintf(fp, " \n"); 
					
					fprintf(fp, "end tell\nend timeout\n"); 
					fclose(fp); 
					}
				
				if (o.testMode) 
					out=system("cat /tmp/tempScript"); 
				else {
					char *reply=NULL; 
					char *scriptstr=readfromfile("/tmp/tempScript"); 
					
					if (scriptstr) 
						reply=callosascriptpoochportd(scriptstr); 
					if (!reply) {//it didn't work, so poochportd might not be running
						out=system("osascript -s s /tmp/tempScript >> /tmp/replyMessage"); 
						//try it the old way
						reply=readfromfile("/tmp/replyMessage"); 
						remove("/tmp/replyMessage"); 
						}
					
					if (reply) {
						
						if (o.rawForm) 
							printf("%s", reply); 
						else {
							if (o.jobScan) 
								parseJobScan(reply, &o); 
							else 
								parseNodeScan(reply, &o); 
						
						if (o.jobScan)
							printf("%d jobs\n", o.numJobs); 
						else 
							printf("%d nodes\n", o.numAvailableNodes); 
						}
						
						free(reply); 
						reply=NULL; 
						}
					
					}
				
				if (!o.saveScript) remove("/tmp/tempScript"); 
				
				
				}
			
			}
		
    	}

	return out; 
	}
