#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* pqstat.c - parallel application queue status  
From OS X's Unix command line, poochstat uses Pooch to 
Accesses entries in the Pooch queue and displays information about them.  
Compile using: sudo cc pqstat.c -o /usr/bin/pqstat

Copyright  2003  Dean E. Dauger, Dauger Research, Inc.  http://daugerresearch.com/

This code requires Pooch version 1.3.5 or later to be installed on the local 
network of Macs.  A download version of Pooch is available at: 
http://daugerresearch.com

****************************************************************
License:

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU General Public License is available by:
Writing to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
or accessing http://www.fsf.org/copyleft/gpl.html

****************************************************************

Version History:

2003 June 18 - Creation using cc on Mac OS X 10.2.5

Thanks goes to the following for advice and recommendations:
  Charles Norton (JPL), Viktor Decyk (UCLA Physics)
*/


void printUsage(); 
void printUsage()
	{
	printf("Accesses the Pooch queue and displays its contents.\n"); 
	printf("Usage: \n\n"); 
	printf("\t jobEntryCode : print out information about a particular job\n"); 
	printf("\t -w : print out all information in wide mode\n"); 
	printf("\t -r : print out raw data rather than a structured format\n"); 
	printf("\t - or --help : print usage information\n"); 
	printf("\n"); 
	}

#include "poochnodescan.h"

typedef struct OptionsStruct {
	short proceed; 
	short saveScript; 
	short wideMode; 
	short testMode;
	short rawForm; 
	short dumpJob; 
	int remoteNodeScanArg; 
	int bestCount; 
	int numAvailableNodes; 
	long flags; 
	NodeExtendedPtr availableNodeList; 
	int unused; 
	} OptionsStruct, *OptionsPtr, **OptionsHandle; 

int parseArguments(int argc, char *argv[], OptionsPtr optionsP); 
int parseArguments(int argc, char *argv[], OptionsPtr optionsP)
	{int out=0; 
	
if (optionsP) {
	{long *tp=(long*)optionsP, i=sizeof(OptionsStruct)>>2; 
	while (i--) tp[i]=0; /* clear it */
	}
	
	if (argc<=0) optionsP->proceed=0; 
	else {
		int i; 
		optionsP->proceed=out=1; 
		
		for(i=1; i<argc; i++) {/* item by item */
			short unrecognizedOption=0, notADashOption=0; 
			
			if ('-'==argv[i][0]) {/* an option */
				char *optS=&argv[i][1]; 
				
				if (strlen(optS)>1) {
#define	ifOptIs(str)	if(!strcmp(optS,str))
					
					ifOptIs("help") {//print help
						optionsP->proceed=0; 
						out=0; 
						}
					else {
						unrecognizedOption=1; 
						printf("Unrecognized option: %s\n", optS); 
						}
					
#undef ifOptIs
					}
				else //single letter or null 
					switch (optS[0]) {
						case '-': //--
						case 0: //-
						case 'h': //-help
						case '?': //-?
							optionsP->proceed=0; 
							out=0; 
							break; 
						case 's': optionsP->saveScript=1; break; 
						case 'w': optionsP->wideMode++; break; 
						case 't': optionsP->testMode=1; break; 
						case 'r': optionsP->rawForm=1; break; 
						case 'a': case '@': 
							if ((i+1<argc)&&!optionsP->remoteNodeScanArg) 
								optionsP->remoteNodeScanArg=i+1; 
							break; 
						case 'n': 
							if ((i+1<argc)&&!optionsP->bestCount) 
								sscanf(argv[i+1], "%d", &optionsP->bestCount); 
							break; 
						case 'j': /* only if first argument */
						default: unrecognizedOption=1; 
							printf("Unrecognized option: %s\n", optS); 
							break; 
					}
				}
			else 
				notADashOption=1; 
			
			if (unrecognizedOption||notADashOption) {
				optionsP->dumpJob=1; 
				/*
				if (optionsP->remoteNodeScanArg) {
					optionsP->remoteNodeScanArg=i; 
					}
				*/
				
				}
			
			}
		}
	
	}
	
	return out; 	
	}

#include "fileutil.c"

#include "poochcljobinput.c"

int displayJobData(JobOptionsPtr o, char *nodeListP); 
int displayJobData(JobOptionsPtr o, char *nodeListP)
	{long i; 
	printf("Job structure is %s.\n", o->proceed?"valid":"invalid"); 
	if (o->numFileArgs>1) {
		printf(" contains %d file(s):\n", o->numFileArgs-1);
		for(i=1; i<o->numFileArgs; i++) 
			printf("%s\n", o->fileArgs[i]); 
		printf(" in the directory context: %s\n", o->fileArgs[0]); 
		}
	if (o->numProcessorOption) 
		printf(" requests %d processors(s)\n", o->numProcessorOption);
	else 
		printf(" requests %d node(s)\n", o->numNodesRequested);
	if (o->subdirectoryOption) printf(" places in directory: %s\n", o->subDirArg);
	if (o->skipBusyCheck) printf(" skips BUSY check\n");
	if (o->launchFailureQueuesJob) printf(" launch failure queues job\n");
	if (o->waitForUserToLaunch) printf(" waits for user to click Launch Job\n");
	if (o->testMode) printf(" runs test mode only\n");
	if (o->saveScript) printf(" saves AppleScript into tempScript\n");
	switch (o->jobType) {
		case 2: printf(" launches MPI/Pro style.\n"); break; 
		case 1: printf(" launches mpich style.\n"); break; 
		default: printf(" launches MacMPI style.\n");
			break; 
		}
	
	if (o->numNodeArgs>1) if (nodeListP) {
		printf("This job was already launched onto %d nodes(s):\n", (o->numNodeArgs-1)>>1);
		printf("%s\n", nodeListP); 
		}
	
	return 1; 
	}

int main(int argc, char *argv[])
	{int out=0; 
	OptionsStruct o; 

	if (!parseArguments(argc, argv, &o)) printUsage(); 
	else {
		int i;
		
		if (o.proceed) 
			{
			long numQueued=0; 
			
			//#include <unistd.h>
			/*
			if (access("/Library/StartupItems/poochdaemon/poochqueue", W_OK|R_OK)) {
				//printf("access failed\n"); 
				}
			else 
				printf("access poochqueue succeeded\n"); 
			*/
			
			if (access("/Library/StartupItems/poochdaemon/poochqueue/in", W_OK|R_OK)) {
				printf("poochqueue/in access failed\n"); 
				o.proceed=0; 
				}
			else {
				if (!o.dumpJob) {
					printf("Pending queue jobs:\n"); 
					system("ls -trl /Library/StartupItems/poochdaemon/poochqueue/in"); 
					printf("\n"); 
					}
				}
			
			if (access("/Library/StartupItems/poochdaemon/poochqueue/out", R_OK)) {
				printf("poochqueue/out access failed\n"); 
				o.proceed=0; 
				}
			else {
				if (!o.dumpJob) {
					printf("Launched queue jobs:\n"); 
					system("ls -tl /Library/StartupItems/poochdaemon/poochqueue/out | head"); 
					printf("\n"); 
					}
				}
			
			if (!o.proceed) {
				printf("The Pooch queue might not be installed properly.\nPlease contact daugerresearch.com for details.\n"); 
				}
			else 
			if (o.dumpJob) {
				long i; 
				for(i=1; i<argc; i++) if ('-'!=argv[i][0]) {
					char *inentry=NULL; 
					char entryName[1024]; 
					strcat(strcpy(entryName, "/Library/StartupItems/poochdaemon/poochqueue/in/"), argv[i]); 
					
					inentry=readfromfile(entryName); 
					if (!inentry) {
						strcat(strcpy(entryName, "/Library/StartupItems/poochdaemon/poochqueue/out/"), argv[i]); 
						inentry=readfromfile(entryName); //try again
						}
					if (inentry) {
						if (!o.rawForm) {
							JobOptionsStruct jobOptions; 
							char *tp; 
							parseJobArgumentsFromData(inentry, &jobOptions); 
							tp=strstr(inentry, "--------\n"); 
							displayJobData(&jobOptions, tp?tp+9:NULL); 
							}
						if (o.rawForm)
							printf("%s\n", inentry); 
						
						free(inentry); 
						}
					else {
						printf("Unable to locate job: %s\n", argv[i]); 
						o.proceed=0; 
						}
					}
				
				}
			
			
			}
		
    	}

	return out; 
	}

