#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* pqsub.c - parallel application submit routine into the Pooch queue 
From OS X's Unix command line, pqsub submits parallel applications 
into the Pooch queue for later submission.  Its options are much like mpirunPooch.
Compile using: sudo cc pqsub.c -o /usr/bin/pqsub

Copyright  2003  Dean E. Dauger, Dauger Research, Inc.  http://daugerresearch.com/

This code requires Pooch version 1.3.5 or later to be installed on the local 
network of Macs.  It translates the parameters entered on the Unix command 
line into AppleScripts that communicate with Pooch.  A download version 
of Pooch is available at: http://daugerresearch.com

****************************************************************
License:

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU General Public License is available by:
Writing to the Free Software Foundation, Inc., 
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
or accessing http://www.fsf.org/copyleft/gpl.html

****************************************************************

Version History:

2003 May 9 - Creation using cc on Mac OS X 10.2.5

Thanks goes to the following for advice and recommendations:
  Charles Norton (JPL), Viktor Decyk (UCLA Physics)
*/


void printUsage(); 
void printUsage()
	{
	printf("Submits jobs to the Pooch queue.\n"); 
	printf("Usage: \n\n"); 
	printf("pqsub executable [file1 file2 ...] -np procCount\t! searches for processors on the \"best\" nodes\n");
	printf("pqsub executable [file1 file2 ...] -n nodeCount\t! searches for the \"best\" nodes\n");
	printf("\nadditional options: \n"); 
	printf("\t -t : test mode on: prints out what I will do, rather than doing it \n"); 
//	printf("\t -a [IP Address] : perform the node scan starting at a remote address\n"); 
	printf("\t -allcpus : include every cpu and node Pooch can find\n"); 
	printf("\t -d : select a directory for the files to be on the remote nodes\n"); 
	printf("\t -T : select a MPI type for the job: -T MacMPI, -T mpich, or -T MPIPro \n"); 
	printf("\t -K : skip the check for nodes being BUSY and launch anyway\n"); 
	printf("\t -Q : in case of a launch failure, Pooch queues the job for later relaunch\n"); 
	printf("\t - or --help : print usage information\n"); 
	printf("\n"); 
	}

typedef struct OptionsStruct {
	short proceed; 
	short wideMode; 
	short testMode;
	short unusedS; 
	char *remotePoochQueueArg; 
	int unused; 
	} OptionsStruct, *OptionsPtr, **OptionsHandle; 

int parseArguments(int argc, char *argv[], OptionsPtr optionsP); 
int parseArguments(int argc, char *argv[], OptionsPtr optionsP)
	{int out=0; 
	
if (optionsP) {
	{long *tp=(long*)optionsP, i=sizeof(OptionsStruct)>>2; 
	while (i--) tp[i]=0; /* clear it */
	}
	
	if (argc<=1) optionsP->proceed=0; 
	else {
		int i; 
		optionsP->proceed=out=1; 
		
		for(i=1; i<argc; i++) {/* item by item */
			short unrecognizedOption=0, notADashOption=0; 
			
			if ('-'==argv[i][0]) {/* an option */
				char *optS=&argv[i][1]; 
				
				if (strlen(optS)>1) {
#define	ifOptIs(str)	if(!strcmp(optS,str))
					
					ifOptIs("help") {//print help
						optionsP->proceed=0; 
						out=0; 
						}
					else {
						unrecognizedOption=1; 
					//	printf("Unrecognized option: %s\n", optS); 
						}
					
#undef ifOptIs
					}
				else //single letter or null 
					switch (optS[0]) {
						case '-': //--
						case 0: //-
						case 'h': //-help
						case '?': //-?
							optionsP->proceed=0; 
							out=0; 
							break; 
				//		case 's': optionsP->saveScript=1; break; 
				//		case 'w': optionsP->wideMode++; break; 
						case 't': optionsP->testMode=1; break; 
				//		case 'r': optionsP->rawForm=1; break; 
						case 'a': case '@': 
							if ((i+1<argc)&&!optionsP->remotePoochQueueArg) 
								optionsP->remotePoochQueueArg=argv[i+1]; 
							break; 
						case 'j': /* only if first argument */
						default: unrecognizedOption=1; 
							//printf("Unrecognized option: %s\n", optS); 
							break; 
					}
				}
			else 
				notADashOption=1; 
			
			if (unrecognizedOption||notADashOption) {
				/*
				if (optionsP->remoteNodeScanArg) {
					optionsP->remoteNodeScanArg=i; 
					}
				*/
				
				}
			
			}
		}
	
	}
	
	return out; 	
	}


#include "fileutil.c"

#include <sys/param.h>
char *getfullpathstr(char *infilestr, char *indirpath)
	{char *out=NULL; 
	
	if (infilestr) {
		if (strlen(infilestr)) {
			char *dirpath=NULL; 
			if (!indirpath) {
				system("pwd >> mydirectorypath.txt"); 
				dirpath=readfromfile("mydirectorypath.txt"); 
				}
			else {
				dirpath=malloc(strlen(indirpath)+1); 
				strcpy(dirpath, indirpath); 
				}
			if (dirpath) {
				char *homepathstr=NULL; 
				remove("mydirectorypath.txt"); 
				switch(infilestr[0]) {
					default: out=(char*)malloc(strlen(dirpath)+strlen(infilestr)+2); 
						if (out) {
							strcpy(out, dirpath); 
							if ('\n'==out[strlen(out)-1]) 
								out[strlen(out)-1]=0; //replace newline char
							if ('/'!=out[strlen(out)-1]) 
								strcat(out, "/"); //add / char
							strcat(out, infilestr); 
							}
						break; 
					case '~': homepathstr=getenv("HOME"); 
						if (homepathstr) {
							out=(char*)malloc(strlen(homepathstr)+strlen(infilestr)+1); 
							if (out) {
								switch (infilestr[1]) {
									case 0: 
										strcpy(out, homepathstr); 
										break; 
									case '/': 
										strcat(strcpy(out, homepathstr), &infilestr[2]); 
										break; 
									default: 
										printf("Sorry, path %s not recognized\n", infilestr); 
										break; 
										}
								
								}
							}
						break; 
					case '/': //already absolute
						out=(char*)malloc(strlen(infilestr)+2); 
						if (out) {
							strcpy(out, infilestr); 
							}
						break; 
					}
				
				{char *fullpathstr=out; 
				out=malloc(MAXPATHLEN); 
				out[0]=0; 
				if (realpath(fullpathstr, out)) 
					free(fullpathstr); 
				else {//put it back
					free(out); 
					out=fullpathstr; 
					}
				}
				
				free(dirpath); 
				dirpath=NULL; 
				}
			}
		}
	
	return out; 
	}

#include "poochcljobinput.c"


#include <time.h>
int submitJobIntoPoochQueue(JobOptionsPtr optionsP)
	{int out=0; 
	if (optionsP) if (optionsP->proceed) {
		FILE *fp; 
		time_t theTime; 
		struct tm calendarTime; 
		char queuePathName[8+14+64], username[L_cuserid]="pooch\0", *loginName=NULL; 
		char queueFileName[32]; 
		
		printf("Proceeding with submission of %s onto ", optionsP->fileArgs[0]); 
		printf(" %d %s%s.\n", 
			optionsP->numProcessorOption?optionsP->numProcessorOption
						:optionsP->numNodesRequested, 
			optionsP->numProcessorOption?"processor":"node", 
			1!=(optionsP->numProcessorOption?optionsP->numProcessorOption
				:optionsP->numNodesRequested)?"s":"\0"); 
		
		//cuserid(username); 
		loginName=getlogin(); 
		if (loginName) 
			strcpy(username, loginName); 
		time(&theTime); 
		localtime_r(&theTime, &calendarTime); 
		sprintf(queueFileName, "%s%02d%02d%02d%02d%02d%02d", 
			username, 
			(calendarTime.tm_year+1900)%100, calendarTime.tm_mon+1, calendarTime.tm_mday, 
			calendarTime.tm_hour, calendarTime.tm_min, calendarTime.tm_sec); 
		
		strcat(strcpy(queuePathName, "/Library/StartupItems/poochdaemon/poochqueue/in/"), 
			queueFileName); 
		
		fp=fopen(queuePathName, "w"); 
		if (fp) {
			long i; 
			fprintf(fp, "\"%s\"\n", optionsP->pathContext); 
			for(i=0; i<optionsP->numFileArgs; i++) {
				fprintf(fp, "%s\n", optionsP->fileArgs[i]); 
				}
			for(i=0; i<optionsP->numExtraArgs; i++) {
				fprintf(fp, "%s\n", optionsP->extraArgs[i]); 
				}
			
			if (optionsP->numProcessorOption) 
				fprintf(fp, "-np %d\n", optionsP->numProcessorOption); 
			else 
				fprintf(fp, "-n %d\n", optionsP->numNodesRequested); 
			
			if (optionsP->nodeArgs[0]) 
				if (!strcmp("65536", optionsP->nodeArgs[0])) 
					fprintf(fp, "-allcpus\n"); 
			
			fprintf(fp, "-d %s\n", optionsP->subDirArg?
				optionsP->subDirArg
				:queueFileName); 
			
			fprintf(fp, "-m\n"); 
			
			if (optionsP->skipBusyCheck) 
				fprintf(fp, "-K\n"); 
			
			if (optionsP->jobType) {
				switch (optionsP->jobType) {
					case 1: 
						fprintf(fp, "-T mpich\n"); 
						break; 
					case 2: 
						fprintf(fp, "-T MPIPro\n"); 
						break; 
					default: 
						break; 
					}
				}
			
			fprintf(fp, "\n"); 
			
			fclose(fp); 
			printf("%s successfully queued in reference code: %s\n", 
				optionsP->fileArgs[0], 
				queueFileName); 
			out=1; 
			}
		else 
			printf("Unable to write queue file.\n"); 
		}					
	return out; 
	}

int main(int argc, char *argv[])
	{int out=0; 
	OptionsStruct o; 

	if (!parseArguments(argc, argv, &o)) printUsage(); 
	else {
		int i; 
		
		if (o.proceed) 
			{char *inqueue=NULL; 
			long numQueued=0; 
			
			if (access("/Library/StartupItems/poochdaemon/poochqueue/in", W_OK|R_OK)) {
				printf("access to poochqueue failed\nThe Pooch queue might not be installed\n"); 
				o.proceed=0; 
				}
			else {
				printf("Pooch queue detected\n"); 
				}
			
			if (o.proceed) {
				JobOptionsStruct jobOptions; 
				
				jobOptions.proceed=0; 
				
				if (!parseJobArgumentsFromMainArgs(argc, argv, &jobOptions)) 
					printUsage(); 
				
				if (jobOptions.proceed) {
					remove("/tmp/mydirectorypath.txt"); 
					system("pwd >> /tmp/mydirectorypath.txt"); 
					//save the path context
					jobOptions.pathContext=readfromfile("/tmp/mydirectorypath.txt"); 
					remove("/tmp/mydirectorypath.txt"); 
					{char *tp=strchr(jobOptions.pathContext, '\n'); 
					if (tp) tp[0]=0; 
					}
					
					if ((!jobOptions.numProcessorOption)&&(!jobOptions.numNodesRequested)) 
						jobOptions.proceed=0; 
					
					if (jobOptions.numFileArgs<=0) 
						jobOptions.proceed=0; 
					
					if (jobOptions.proceed) {
						submitJobIntoPoochQueue(&jobOptions); 
						}
					
					
					}
				
				
				}
			
			}
		
    	}

	return out; 
	}
